# source /apps/lab/aryee/pyenv/versions/venv-2.7.10/bin/activate

import HTSeq
import re
import itertools

#fa1 = "/data/rivera/Data/07.31.15.Nextseq/tmp.r1.fasta"
#fa2 = "/data/rivera/Data/07.31.15.Nextseq/tmp.r2.fasta"


fq = "/data/rivera/Data/07.31.15.Nextseq/SKNMC.cohesin.chiapet.test.fastq"
fa1 = "/data/rivera/Data/07.31.15.Nextseq/SKNMC.cohesin.chiapet.test.extracted.r1.fasta"
fa2 = "/data/rivera/Data/07.31.15.Nextseq/SKNMC.cohesin.chiapet.test.extracted.r2.fasta"

fq = "/data/rivera/Data/07.31.15.Nextseq/SKNMC.FLI1.chiapet.test.fastq"
fa1 = "/data/rivera/Data/07.31.15.Nextseq/SKNMC.FLI1.chiapet.test.extracted.r1.fasta"
fa2 = "/data/rivera/Data/07.31.15.Nextseq/SKNMC.FLI1.chiapet.test.extracted.r2.fasta"


fastq_file = HTSeq.FastqReader(fq)
out_fasta_file1 = open(fa1, 'w')
out_fasta_file2 = open(fa2, 'w')

red = 0
blue = 0
nomatch = 0

#for read in itertools.islice( fastq_file, 10):
for read in fastq_file:
    #print read.name
    #print read.seq
    m_blue = re.search('(.+)GTTGGATAAGATATCGCGGCCGCGATATCTTATCCAAC(.{18}).*', read.seq)
    m_red = re.search('(.+)GTTGGAATGTATATCGCGGCCGCGATATACATTCCAAC(.{18}).*', read.seq)
    m = None
    if m_blue:
        #print "Blue"
        blue = blue + 1
        m = m_blue
    elif m_red:
        #print "Red"
        red = red + 1        
        m = m_red
    else:
        #print "No match"
        nomatch = nomatch + 1
    if m:
        #print m.group(1)
        #print m.group(2)
        read.seq = m.group(1)
        read.write_to_fasta_file( out_fasta_file1 )
        read.seq = m.group(2)        
        read.write_to_fasta_file( out_fasta_file2 )    



out_fasta_file1.close()
out_fasta_file2.close()

print red + blue + nomatch, "total reads"
print red, "red reads"
print blue, "blue reads"
print nomatch, "no match reads"







import pytest
from click.testing import CliRunner
from dnaloop import cli
import md5

def file_checksums_equal(file1, file2):
    with open(file1) as f:
        checksum1 = md5.new(f.read()).digest()
    with open(file2) as f:
        checksum2 = md5.new(f.read()).digest()
    return checksum1==checksum2    

#@pytest.fixture
  
def test_preproc_run():
    runner = CliRunner()
    # Testing: preprocess_chiapet --out output1 --bwa-index test_genome.fa --keep-temp-files samples.txt
    result = runner.invoke(cli.main, ['--out', 'output1', '--bwa-index', 'test_genome.fa', '--keep-temp-files', '--no-qc-report', 'samples.txt'])
    assert not result.exception
    assert result.exit_code == 0

def test_reads_for_peak_calling():
    assert file_checksums_equal('correct_output/peaks/reads.bed', 'output1/peaks/reads.bed')

def test_peaks():
    assert file_checksums_equal('correct_output/peaks/anchor_peaks.narrowPeak', 'output1/peaks/anchor_peaks.narrowPeak')
    assert file_checksums_equal('correct_output/peaks/anchor_peaks.merged.bed', 'output1/peaks/anchor_peaks.merged.bed')
    
def test_loop_counts():
    assert file_checksums_equal('correct_output/naive_esc_1.loop_counts.bedpe', 'output1/naive_esc_1.loop_counts.bedpe')

def test_preproc_run_mergegap():
    runner = CliRunner()
    result = runner.invoke(cli.main, ['--out', 'output2', '--bwa-index', 'test_genome.fa', '--merge-gap', '1000', '--no-qc-report', 'samples.txt'])
    assert not result.exception
    assert result.exit_code == 0

def test_parse_yaml_manifest():
    samples = [ {   'name': 'naive_esc',
                    'read1': 'fastq/naive_esc_1.r1.fastq.gz,fastq/naive_esc_2.r1.fastq.gz', 
                    'read2': 'fastq/naive_esc_1.r2.fastq.gz,fastq/naive_esc_2.r2.fastq.gz'
                },
                {   'name': 'primed_esc',
                    'read1': 'fastq/primed_esc_1.r1.fastq.gz,fastq/primed_esc_2.r1.fastq.gz', 
                    'read2': 'fastq/primed_esc_1.r2.fastq.gz,fastq/primed_esc_2.r2.fastq.gz'

                }]
    assert cli.parse_manifest('samples.yaml') == samples



import click
import os
import shutil
import yaml
import shutil
import random
import string
from pkg_resources import get_distribution
from subprocess import call, check_call
import pandas as pd
import numpy as np
import matplotlib
matplotlib.use('Agg')
from matplotlib.backends.backend_pdf import PdfPages
import matplotlib.pyplot as plt

def parse_manifest(manifest):
    samples = []
    if manifest.endswith(('.yaml', '.yml')):
        with open(manifest, 'r') as f: 
            m = yaml.load(f)
        sample_names = m['samples'].keys()
        sample_names.sort()
        for sample_name in sample_names:
            runs = m['samples'][sample_name]
            read1 = []
            read2 = []
            for run in runs:
                fastq1, fastq2 = run.split(" ")
                read1.append(fastq1)
                read2.append(fastq2)
            d = {
                'name': sample_name, 
                'read1': ','.join(read1),
                'read2': ','.join(read2)
                }
            samples.append(d)
        return samples
    else:
        with open(manifest) as f:
            lines = f.readlines()
        for line in lines:
            fields = line.strip().split("\t")
            if len(fields)==3:
                d = {   'name': fields[0], 
                        'read1': fields[1],
                        'read2': fields[2]
                        }
                samples.append(d)
            else:
                if line != "\n":
                    click.echo ("Skipping line: " + line)
        return samples

#Grab Sample Names
def get_subdirectories(dir):
    return [name for name in os.listdir(dir)
            if os.path.isdir(os.path.join(dir, name))]

def qc_report(dir):
    #Read in the summary statistics            
    statslist = []
    sampleNames = get_subdirectories(os.path.join(dir, "samples"))
    for s in sampleNames:
        sfile = open(os.path.join(dir, "samples/" + s + "/read_stats.txt"), 'r')
        stats = []
        header = sfile.readline()
        for l in sfile.readlines():
            stats.append(int(l.split(": ")[1].strip()))	
        statslist.append(stats)
    
    readstats = pd.DataFrame(statslist)
    readstats = readstats.transpose()
    readstats.columns = sampleNames
    
    # Now make the plots
    with PdfPages(os.path.join(dir, 'qc-report.pdf')) as pdf:    
        plt.figure(figsize=(8, 11))
        readstats.ix[0].plot(kind='bar')
        plt.title('Total Reads Per Sample')
        pdf.savefig()  # saves the current figure into a pdf page
        plt.close()


@click.command()
#@click.option('--cluster-command', '-c', default='bsub', help='Cluster submit command')
@click.option('--out', default=".", required=True, help='Output directory name')
@click.option('--bwa-index', required=True, help='BWA index location')
@click.option('--peak-pad', default="0", help='Peak padding width (applied on both left and right)')
@click.option('--merge-gap', default="1500", help='Max gap size for merging peaks')
@click.option('--use-lsf', is_flag=True, help='Submit jobs to an LSF cluster?')
@click.option('--bsub-opts', default="", help='LSF bsub options')
@click.option('--keep-temp-files', is_flag=True, help='Keep temporary files?')
@click.option('--no-qc-report', is_flag=True, help='Skip QC report generation? (Requires R)')
@click.argument('manifest')
#def main(manifest, cluster):
def main(manifest, out, bwa_index, peak_pad, merge_gap, use_lsf, bsub_opts, keep_temp_files, no_qc_report):
    """A preprocessing and QC pipeline for ChIA-PET data."""
    __version__ = get_distribution('dnaloop').version
    click.echo("Starting dnaloop pipeline v%s" % __version__)
    if os.path.exists(out):
        shutil.rmtree(out)
    os.mkdir(out)
    os.mkdir(os.path.join(out, 'log'))
    with open(os.path.join(out, 'log', 'VERSION.txt'), 'w') as f: 
        f.write(__version__ + '\n')
    script_dir = os.path.dirname(os.path.realpath(__file__))
    out = os.path.abspath(out)    
    bwa_index = os.path.abspath(bwa_index)    
    click.echo("Output folder: %s" % out) 
    click.echo("BWA index: %s\n" % bwa_index)     
    # Preprocess individual samples
    samples = parse_manifest(manifest)
    i = 0
    if use_lsf:
        job_ids = []
        lsf_id = ''.join(random.SystemRandom().choice(string.ascii_lowercase + string.digits) for _ in range(5))
    for sample in samples:
        i += 1
        click.echo("\nProcessing sample %d of %d: %s" % (i, len(samples), sample['name']))
        click.echo("    Read 1: %s" % sample['read1']) 
        click.echo("    Read 2: %s" % sample['read2'])    
        preproc_fastq = os.path.join(script_dir, 'preprocess_chiapet_fastq.sh')
        cmd = [preproc_fastq, os.path.join(out, 'samples', sample['name']), bwa_index, merge_gap, sample['read1'], sample['read2']]
        if use_lsf:
            job_id = 'dnaloop_sample_%s_%d' % (lsf_id, i)
            cmd = "bsub -J %s %s %s" % (job_id, bsub_opts,  " ".join(cmd))
            click.echo("    Submitting job to LSF: %s" % cmd)    
            check_call(cmd, shell=True)
            job_ids.append(job_id)        
        else:
            click.echo("    Executing: %s" % " ".join(cmd))
            call(cmd)
    if use_lsf:
        conditions = ["ended(%s)" % job_id for job_id in job_ids]
        depend_cond = " && ".join(conditions)
        cmd = 'bsub -K -o /dev/null -q long -w "%s" exit 0' % depend_cond
        print("Waiting for %d dnaloop sample preprocessing jobs to finish" % len(job_ids))
        check_call(cmd, shell=True)        
    # Create the ChIA-PET analysis set
    preproc_set = os.path.join(script_dir, 'preprocess_chiapet_set.sh')
    cmd = [preproc_set, out, peak_pad, merge_gap] + [os.path.join(out, 'samples', x['name']) for x in samples]
    click.echo("Creating ChIA-PET set")
    click.echo("    Executing: %s\n" % " ".join(cmd))
    call(cmd)
    if no_qc_report:
        click.echo("Skipping QC report since --no-qc-report was specified")
    else:
        click.echo("Creating QC report")
        cmd = ['Rscript', os.path.join(script_dir, 'qc-report.R'), out] 
        call(cmd)
    if keep_temp_files:
        click.echo("Temporary files not deleted since --keep-temp-files was specified")
    else:
        click.echo("Deleting temporary files")
        shutil.rmtree(os.path.join(out, 'peaks'))
        shutil.rmtree(os.path.join(out, 'samples'))
    click.echo("Done")





import subprocess                                     
import os
import pandas as pd
import numpy as np
from matplotlib.backends.backend_pdf import PdfPages
import matplotlib.pyplot as plt

#Grab Sample Names
def get_subdirectories(dir):
    return [name for name in os.listdir(dir)
            if os.path.isdir(os.path.join(dir, name))]

#Read in the summary statistics            
statslist = []
sampleNames = get_subdirectories("samples/")
for s in sampleNames:
	sfile = open("samples/" + s + "/read_stats.txt", 'r')
	stats = []
	header = sfile.readline()
	for l in sfile.readlines():
		stats.append(int(l.split(": ")[1].strip()))	
	statslist.append(stats)
readstats = pd.DataFrame(statslist)
readstats = readstats.transpose()
readstats.columns = sampleNames

#Parse .bedpe files
for s in sampleNames:
	sfilename = s + ".loop_counts.bedpe"
	dist_counts = subprocess.check_output("awk '$1 == $4 {print $5-$2 \"\t\" $8}' " + sfilename, shell=True)
	print dist_counts

# Now make the plots
with PdfPages('qc-report.pdf') as pdf:

    plt.figure(figsize=(8, 11))
    readstats.ix[0].plot(kind='bar')
    plt.title('Total Reads Per Sample')
    
    pdf.savefig()  # saves the current figure into a pdf page
    plt.close()

    plt.rc('text', usetex=True)
    plt.figure(figsize=(3, 3))
    x = np.arange(0, 5, 0.1)
    plt.plot(x, np.sin(x), 'b-')
    plt.title('Page Two')

    pdf.savefig()
    plt.close()




